@@ -3,7 +3,7 @@ package Promise::ES6;
 use strict;
 use warnings;
 
-our $VERSION = '0.02-TRIAL1';
+our $VERSION = '0.03-TRIAL1';
 
 =encoding utf-8
 
@@ -84,6 +84,7 @@ sub then {
     my $new = {
         _on_resolve => $on_resolve,
         _on_reject => $on_reject,
+        _warned_unhandled_reject => $self->{'_warned_unhandled_reject'},
     };
 
     bless $new, (ref $self);
@@ -115,6 +116,12 @@ sub _finish {
 
     if ($self->{"_on_$how"}) {
         if ( eval { $value = $self->{"_on_$how"}->($value); 1 } ) {
+
+            # This is here so that a rejection that’s caught
+            # after-the-fact will “reset”. That way further promises
+            # in the chain will warn() on rejection.
+            delete $self->{'_warned_unhandled_reject'};
+
             $how = 'resolve';
         }
         else {
@@ -142,6 +149,13 @@ sub _finish {
             $self->{'_finished_how'} = $repromise_how;
 
             $_->_finish($repromise_how, $repromise_value) for @{ $self->{'_dependents'} };
+
+            if ($repromise_how eq 'reject' && !@{ $self->{'_dependents'} }) {
+                $self->{'_warned_unhandled_reject'} ||= do {
+                    my $ref = ref $self;
+                    warn "$ref: Unhandled rejection: $repromise_value";
+                };
+            }
         }
     };
 
